@@ -154,6 +154,9 @@ hd_wm_process_x_client_list (HDWM *hdwm);
 static gboolean
 hd_wm_relaunch_timeout (gpointer data);
 
+static void
+hd_wm_monitor_register (HDWM *hdwm);
+
 static void 
 hd_wm_register_object_path (HDWM *hdwm,
  			    DBusConnection *conn,
@@ -831,6 +834,8 @@ hd_wm_constructor (GType gtype, guint n_params, GObjectConstructParam *params)
     dbus_connection_flush (connection);
   }
 
+  hd_wm_monitor_register (hdwm);
+
   if (!sys_connection)
   {
       hd_wm_debug ("Failed to connect to DBUS: %s!\n", sys_error.message );
@@ -1138,6 +1143,8 @@ hd_wm_init (HDWM *hdwm)
   
   /* build our hash of watchable apps via .desktop key/values */
 
+  hdwm->priv->monitor_timeout_id = 0;
+  
   hdwm->priv->applications = NULL;
   
   hdwm->priv->watched_apps = hd_wm_applications_init ();
@@ -3154,30 +3161,25 @@ hd_wm_monitor_cb (GnomeVFSMonitorHandle *handle,
                   GnomeVFSMonitorEventType event_type,
                   gpointer user_data)
 {
-  HDWM *hdwm = hd_wm_get_singleton ();
-	
+  HDWM *hdwm = HD_WM (user_data);
+  
   if (!hdwm->priv->monitor_timeout_id)
     {
       hdwm->priv->monitor_timeout_id =
-      g_timeout_add(1000,
-                    (GSourceFunc)hd_wm_monitor_process,
-                    NULL);
+      g_timeout_add (1000,
+                     (GSourceFunc)hd_wm_monitor_process,
+                     (gpointer) hdwm);
     }
 }
 
-/* Public function to register our monitor callback (called from
- * hildon-navigator-main.c)
- */
-void
-hd_wm_monitor_register ()
+static void
+hd_wm_monitor_register (HDWM *hdwm)
 {
-  HDWM *hdwm = hd_wm_get_singleton ();
-
   if (gnome_vfs_monitor_add (&hdwm->priv->monitor, 
                              DESKTOPENTRYDIR,
                              GNOME_VFS_MONITOR_DIRECTORY,
                              (GnomeVFSMonitorCallback) hd_wm_monitor_cb,
-                             NULL) != GNOME_VFS_OK)
+                             hdwm) != GNOME_VFS_OK)
   {
     g_warning("unable to register TN callback for %s", DESKTOPENTRYDIR);
   }
